import { Component, Input, ViewChild, EventEmitter, ContentChild, Output } from '@angular/core';
import { NNInternalTreeNode } from './nn-internal-tree-node';
var NNTreeComponent = (function () {
    function NNTreeComponent() {
        this.selectParent = false;
        this.lazyLoading = false;
        this.loadingText = "Loading...";
        this.loadingIcon = "";
        this.showRoot = true;
        this.height = 100;
        this.changeselection = new EventEmitter();
        this.openNode = new EventEmitter();
        this.closeNode = new EventEmitter();
        this.renderNodes = [];
        this.displayNodes = [];
        this.orginalNodes = [];
        this.itemHeight = 20;
        this.itemsPerViewport = 5;
        this.startIndex = 0;
        this.actualHeight = 0;
        this.filterText = "";
        this.paddingLeft = 20;
        this.numDisplayItems = 0;
        this._index = 0;
    }
    NNTreeComponent.prototype.ngOnInit = function () { };
    Object.defineProperty(NNTreeComponent.prototype, "root", {
        set: function (root) {
            if (this._root !== root) {
                this._root = root;
                root.display = this.showRoot;
                this.initTreeData(root, 0);
                this.actualHeight = this.displayNodes.length * this.itemHeight;
                this.itemsPerViewport = Math.floor(this.height / this.itemHeight) + 2;
                this.renderNodes = new Array(this.itemsPerViewport);
                this.refresh(true);
            }
        },
        enumerable: true,
        configurable: true
    });
    NNTreeComponent.prototype.initTreeData = function (node, level) {
        var _this = this;
        node.left = level * this.paddingLeft - (!this.showRoot ? this.paddingLeft : 0);
        node.level = level;
        node.isLeaf = node.lazyLoading ? true : (node.children && node.children.length > 0) ? true : false;
        if (node.display) {
            this.displayNodes.push(node);
        }
        this.orginalNodes.push(node);
        if (node.children && node.children.length)
            node.children.forEach(function (n) {
                n.display = node.open;
                _this.initTreeData(n, level + 1);
            });
    };
    NNTreeComponent.prototype.refresh = function (force) {
        var showFromIndex = Math.floor(this.container.nativeElement.scrollTop / this.itemHeight);
        if (this.startIndex != showFromIndex || force) {
            this.startIndex = force ? this.startIndex : showFromIndex;
            var count = 0;
            for (var i = this.startIndex; i < this.displayNodes.length && count < this.renderNodes.length; i++) {
                this.renderNodes[count] = this.displayNodes[i];
                this.renderNodes[count].index = i;
                this.renderNodes[count].top = 0;
                count++;
            }
            this.renderNodes[0].top = this.startIndex * this.itemHeight;
            this.numDisplayItems = count;
        }
    };
    NNTreeComponent.prototype.onClickToogleIcon = function (node) {
        node.open = !node.open;
        node.open ? this.onOpenNode(node) : this.onCloseNode(node);
    };
    NNTreeComponent.prototype.onOpenNode = function (node) {
        node.open = true;
        if (!this.lazyLoading) {
            this.addRenderedChildren(node, node.children);
        }
        this.openNode.emit(node);
    };
    NNTreeComponent.prototype.onCloseNode = function (node) {
        node.open = false;
        node.children.forEach(function (x) { return x.display = false; });
        this.removeRenderedChildren(node);
        this.closeNode.emit(node);
    };
    NNTreeComponent.prototype.onClickItem = function (node) {
        if (node.children && node.children.length > 0 && !this.selectParent) {
            return;
        }
        if (this.selectedNode != node) {
            if (this.selectedNode)
                this.selectedNode.selected = false;
            this.selectedNode = node;
            this.selectedNode.selected = true;
            this.changeselection.emit(node);
        }
    };
    NNTreeComponent.prototype.onDblclickItem = function (node) {
        if (node.children && node.children.length)
            this.onClickToogleIcon(node);
    };
    NNTreeComponent.prototype.filter = function (text) {
        this.filterText = text;
        this.displayNodes = [];
        var indexs = [];
        var currentLevel = 0;
        for (var i = this.orginalNodes.length - 1; i >= 0; i--) {
            var node = this.orginalNodes[i];
            if (node.label.search(text) >= 0) {
                if (node.display)
                    indexs.push(i);
                currentLevel = node.level;
            }
            else {
                if (node.level < currentLevel) {
                    if (node.display)
                        indexs.push(i);
                    currentLevel = node.level;
                }
            }
        }
        this.displayNodes = new Array(indexs.length);
        var count = 0;
        for (var i = indexs.length - 1; i >= 0; i--) {
            if (this.orginalNodes[indexs[i]].display)
                this.displayNodes[count++] = this.orginalNodes[indexs[i]];
        }
        this.actualHeight = this.displayNodes.length * this.itemHeight;
        this.refresh(true);
    };
    NNTreeComponent.prototype.showLoading = function (node) {
        node.children = [];
        this.removeRenderedChildren(node);
        var n = new NNInternalTreeNode();
        n.open = true;
        n.loading = true;
        this.addRenderedChildren(node, [n]);
    };
    NNTreeComponent.prototype.hideLoading = function (node) {
        this.removeRenderedChildren(node);
        node.children = [];
    };
    NNTreeComponent.prototype.addRenderedChildren = function (node, children, noRefresh) {
        var _this = this;
        node.children = children;
        this._index = node.index;
        var count = 0;
        if (children && children.length) {
            children.forEach(function (x, i) {
                _this._index++;
                x.display = node.open;
                x.level = node.level + 1;
                x.index = _this._index;
                x.left = node.left + _this.paddingLeft;
                x.top = node.top + (count + 1) * _this.itemHeight;
                if (x.loading || (x.label != undefined && x.label.search(_this.filterText)) >= 0) {
                    _this.displayNodes.splice(x.index, 0, x);
                    _this.actualHeight += _this.itemHeight;
                }
                if (x.open && x.children && x.children.length) {
                    _this.addRenderedChildren(x, x.children, true);
                }
            });
        }
        if (!noRefresh)
            this.refresh(true);
    };
    NNTreeComponent.prototype.removeRenderedChildren = function (node) {
        var count = 0;
        for (var i = node.index + 1; i < this.displayNodes.length; i++) {
            if (node.level < this.displayNodes[i].level)
                count++;
            else
                break;
        }
        this.actualHeight -= count * this.itemHeight;
        this.displayNodes.splice(node.index + 1, count);
        this.refresh(true);
    };
    NNTreeComponent.prototype.addNodeChildren = function (node, children) {
        if (children) {
            children.forEach(function (n) {
                n.isLeaf = n.lazyLoading ? true : (n.children && n.children.length > 0) ? true : false;
            });
        }
        this.addRenderedChildren(node, children);
    };
    NNTreeComponent.prototype.removeNodeChildren = function (node) {
        node.children = [];
        this.removeRenderedChildren(node);
    };
    return NNTreeComponent;
}());
export { NNTreeComponent };
NNTreeComponent.decorators = [
    { type: Component, args: [{
                selector: 'nn-tree',
                templateUrl: './nn-tree.component.html',
                styleUrls: ['./nn-tree.component.css']
            },] },
];
/** @nocollapse */
NNTreeComponent.ctorParameters = function () { return []; };
NNTreeComponent.propDecorators = {
    'width': [{ type: Input },],
    'selectParent': [{ type: Input },],
    'lazyLoading': [{ type: Input },],
    'loadingText': [{ type: Input },],
    'loadingIcon': [{ type: Input },],
    'showRoot': [{ type: Input },],
    'height': [{ type: Input },],
    'changeselection': [{ type: Output },],
    'openNode': [{ type: Output },],
    'closeNode': [{ type: Output },],
    'nnTreeItem': [{ type: ContentChild, args: ["nnTreeItem",] },],
    'nnTreeToogleIcon': [{ type: ContentChild, args: ["nnTreeToogleIcon",] },],
    'container': [{ type: ViewChild, args: ["container",] },],
    'root': [{ type: Input },],
};
//# sourceMappingURL=nn-tree.component.js.map